<?php declare(strict_types=1);

namespace Plugin\jtl_trustami_produkt_siegel;

use JTL\Events\Dispatcher;
use JTL\Plugin\Bootstrapper;
use JTL\Shop;
use JTL\Smarty\JTLSmarty;

/**
 * Class Bootstrap
 * @package Plugin\jtl_trustami_produkt_siegel
 */
class Bootstrap extends Bootstrapper
{
    /**
     * @var TestHelper
     */
    private $helper;


    /**
     * @inheritdoc
     */
    public function boot(Dispatcher $dispatcher): void
    {
        parent::boot($dispatcher);

        $plugin = $this->getPlugin();
        if (Shop::isFrontend()) {
            $this->helper = new Helper($plugin);
            
            $dispatcher->listen('shop.hook.' . \HOOK_ARTIKEL_PAGE, function (array $args) use ($plugin) {
                $config = $plugin->getConfig();
                if (!empty($plugin->getConfig()->getValue('apiUserName'))) {
                    $taCfg = [];
                    $gtin = $this->helper->getGtinFromArtikel($args['oArtikel']);
                    $this_article = !empty($args) && !empty($args['oArtikel']) ? $args['oArtikel'] : false;
                    if ($this_article) $filteredProducts = [$this_article];
                    else $filteredProducts = [];

                    if (!empty($gtin)) {
                        $taModes = array();

                        $prodViewTtlBox = $plugin->getConfig()->getValue('prodViewTtlBox');
                        $prodViewRtgBox = $plugin->getConfig()->getValue('prodViewRtgBox');
                        if ($prodViewTtlBox != "0") {
                            $taModes[] = intval($prodViewTtlBox);
                        }
                        if ($prodViewRtgBox != "0") {
                            $taModes[] = intval($prodViewRtgBox);
                            if ($prodViewRtgBox == "4.5") {
                                $taCfg[] = '"4":{"nbCol":2}';
                            } 
                        }                        

                        if (!empty($taModes)) {
                            if (!empty($taCfg)) {
                                Shop::Smarty()->assign('taCfg', urlencode('{'.join(',', $taCfg).'}'));
                            }
                            if (!empty($plugin->getConfig()->getValue('customDataAttr'))) {
                                Shop::Smarty()->assign("cusDataAttr", urlencode($plugin->getConfig()->getValue('customDataAttr')));
                                $dataAttr = [];
                                $parsedJsonAttr = json_decode($plugin->getConfig()->getValue('customDataAttr'));
                                foreach($parsedJsonAttr as $key => $value) {
                                  $dataAttr[] = $key.'="'.$value.'"';
                                }
                                Shop::Smarty()->assign('dataAttr', implode(" ",$dataAttr));
                            }
                            if ($config->getValue('prodShopRtgCnt') != "0")
                                Shop::Smarty()->assign('prodParam', $this->helper->assignScriptParam($args['oArtikel']));
                            Shop::Smarty()->assign('prodGtin', $gtin);
                            Shop::Smarty()->assign('taModes', urlencode('['.join(',', $taModes).']'));
                            Shop::Smarty()->assign('prodUserId', $plugin->getConfig()->getValue('apiUserName'));
                            Shop::Smarty()->assign('prodViewTtlBox', $prodViewTtlBox);
                            Shop::Smarty()->assign('prodViewRtgBox', $prodViewRtgBox);
                        }
                    }
                    Shop::Smarty()->assign('prodZeroStars', $plugin->getConfig()->getValue('prodZeroStars'));
                    if(!empty(Shop::Smarty()->getTemplateVars('oAehnlicheArtikel_arr'))) {
                        $filteredProducts = array_merge($filteredProducts, Shop::Smarty()->getTemplateVars('oAehnlicheArtikel_arr'));
                    }         
                    $this->setTrustamiProductReviews(
                        $config, 
                        $filteredProducts, 
                        'trustamiProducts', 
                        ($this_article ? ['this_article' => $this_article] : null)
                    );
                }
            });

            $dispatcher->listen('shop.hook.' . \HOOK_FILTER_PAGE, function (array $args) use ($plugin) {
                if ($plugin->getConfig()->getValue('prodCatSearchBox') != "0" && Shop::Smarty()->getTemplateVars('Suchergebnisse') != NULL)
                    $this->setTrustamiProductReviews($plugin->getConfig(), Shop::Smarty()->getTemplateVars('Suchergebnisse')->getProducts());
            });

            $dispatcher->listen('shop.hook.' . \HOOK_BOXEN_HOME, function (array $args) use ($plugin) {
                if ($plugin->getConfig()->getValue('prodStartPageBox') != "0") {
                    foreach ($args["boxes"] as $box) {
                        if ($box->name == 'TopAngebot' && isset($box->Artikel)) {
                            $this->setTrustamiProductReviews($plugin->getConfig(), $box->Artikel->elemente);
                            break;
                        }
                    }
                }
            });

            $dispatcher->listen('shop.hook.' . \HOOK_WARENKORB_PAGE, function (array $args) use ($plugin) {
                $productCart = Shop::Smarty()->getTemplateVars('Warenkorb')->PositionenArr;
                if ($plugin->getConfig()->getValue('prodCartBox') != "0" && !empty($productCart)) {
                    $filteredItems = array();
                    foreach ($productCart as $cprod) {
                        if(isset($cprod->Artikel)) {
                            array_push($filteredItems, $cprod->Artikel);
                        }
                    }
                    $this->setTrustamiProductReviews($plugin->getConfig(), $filteredItems);
                }
            });

            $dispatcher->listen('shop.hook.' . \HOOK_INDEX_NAVI_HEAD_POSTGET, function (array $args) use ($plugin) {
                if ($plugin->getConfig()->getValue('prodCmpBox') != "0" && Shop::Smarty()->getTemplateVars('oVergleichsliste')) {
                    $this->setTrustamiProductReviews($plugin->getConfig(), Shop::Smarty()->getTemplateVars('oVergleichsliste')->oArtikel_arr);
                }
            });

            $dispatcher->listen('shop.hook.' . \HOOK_SMARTY_INC, function (array $args) use ($plugin) {
                if ($plugin->getConfig()->getValue('prodWishBox') != "0" && isset($_SESSION["Wunschliste"])) {
                    $wishList = array();
                    foreach ($_SESSION["Wunschliste"]->CWunschlistePos_arr as $classElement) {
                        if (isset($classElement->Artikel)){
                            array_push($wishList, $classElement->Artikel);
                        }  
                    }
                    $this->setTrustamiProductReviews($plugin->getConfig(), $wishList, "trustamiProdWishList");
                }
            });
        }
    }

    public function setTrustamiProductReviews($config, $filteredProducts, $smartyName='trustamiProducts', $additional_vars = null): void
    {
        $apiUserName = $config->getValue('apiUserName');
        $apiKey = $config->getValue('apiKey');
        //allways add prodZeroStars to smarty in order to print also zero star ratings if applicable
        Shop::Smarty()->assign('prodZeroStars', $config->getValue('prodZeroStars'));
        if ($additional_vars) {
            foreach($additional_vars as $sName => $value) {
                Shop::Smarty()->assign($sName, $value);
            }
        }

        if (!empty($apiKey) && !empty($apiUserName) && !empty($filteredProducts)) {
            $apiCred = $apiUserName.":".$apiKey;
            $apiPostItems = $this->helper->buildApiPostObjects($filteredProducts, $config->getValue('prodShopRtgCnt'));
            if (!empty($apiPostItems)) {
                $payload = ["params" => $apiPostItems];
                $taClient = new TrustamiProductApi($apiCred);
                $results = $taClient->postMultipleProducts($payload);
                if ($results) {
                    $jsonResponse = json_decode($results);
                    if (isset($jsonResponse->code) && $jsonResponse->code == 200 && isset($jsonResponse->result)){
                        Shop::Smarty()->assign($smartyName, $jsonResponse->result);
                    }
                }
            }  
        }
    }

}